/*
* file CcView.java
*
* Licensed Materials - Property of IBM
* Restricted Materials of IBM
* 
* com.ibm.rational.wvcm.stp.cc.CcView
*
* (C) Copyright IBM Corporation 2004, 2012.  All Rights Reserved. 
* Note to U.S. Government Users Restricted Rights:  Use, duplication or  
* disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
*/

package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.io.File;
import java.util.List;
import java.util.Map;

import javax.wvcm.Feedback;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList;
import javax.wvcm.Stream;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpActivity;
import com.ibm.rational.wvcm.stp.cc.CcFileAreaLockedCallback.CcFileAreaLockInfo;
import com.ibm.rational.wvcm.stp.cc.CcViewTag.ViewType;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;

/**
 * <p>A proxy for a ClearCase view.  ClearCase "view" and WVCM "workspace"
 * are equivalent terms for the same type of resource. 
 * </p>
 * <p>As of the ClearCase 8.0 release, CM API supports web views and dynamic
 * views.  ClearCase snapshot views are not currently supported.
 * </p>
 */
public interface CcView 
    extends CcDirectory, javax.wvcm.Workspace
{
    /** Values for view text mode */
    enum TextMode implements StpExEnumeration {
        
        /**
         * A transparent interop text mode. The line terminator for text files 
         * is a single <code>NL</code> character. The view does not transform 
         * text file line terminators in any way. This is the default text mode 
         * if no value is set upon view creation.
         */
        TRANSPARENT("transparent"),
        
        /**
         * An insert_cr interop text mode. The view converts <code>NL</code> line 
         * terminators to the <code>CR NL</code> sequence when reading from a VOB,
         * and <code>CR NL</code> line terminators to single <code>NL</code>
         * characters when writing to the VOB.
         */
        INSERT_CR("insert_cr"),
        
        /**
         * An nl_to_cr interop text mode. The view converts <code>NL</code> line
         * terminators to <code>CR</code> characters when reading from a VOB, and
         * <code>CR</code> line terminators to single <code>NL</code> characters
         * wehn writing to the VOB.
         */
        NL_TO_CR("cvt_nl_to_cr"),
        
        /**
         * A strip_cr interop text mode. The view converts <code>CR NL</code> line 
         * terminators to <code>NL</code> when reading from a VOB, and <code>NL</code> 
         * line terminators back to the <code>CR NL</code> sequence when writing to the VOB.
         */
        STRIP_CR("strip_cr");
        
        private String m_name;

        private TextMode(String name) { m_name = name; }

        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString() { return m_name; }
    }
    
    /** Flags for the <code>doSynchronizeFileAreaDb</code> method */
    enum SynchronizeFileAreaDbFlag implements StpExEnumeration {
        
        /**
         * Synchronize the web view's client-side vob database with the
         * correct information from the server. Specifying this flag
         * requires a server connection.
         */
        FILE_AREA_VOB_DB("file-area-vob-db"),
        
        /**
         * Synchronize the web view's client-side modified files database
         * so that it correctly reflects files that have been modified
         * as a result of checkouts or hijacks. Specifying this flag
         * does <i>not</i> require a server connection.
         */
        FILE_AREA_MODIFIED_FILES_DB("file-area-modified-files-db");
    
        private String m_name;

        private SynchronizeFileAreaDbFlag(String name) { m_name = name; }

        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString() { return m_name; }
    }
    
    /** Flags for the doFindMergeCandidates methods */
    enum FindmergeFlag {
        
        /**
         * Just search the directory itself when searching for merge candidates,
         * do not search the child elements
         */
        DIRECTORY_ONLY,        
        
        /**
         * Follow VOB symbolic links when searching for merge candidates
         */
        FOLLOW_SYMLINKS                
    }
    
    /**
     *<p>
     * Binds a CcActivity proxy to a CqRecord proxy without making the
     * activity the current one for this view.
     * </p>
     * <p>
     * NOTE: Unlike most API methods, the optional property request will be executed
     * on the returned StpActivity, not the CcView proxy on which the method
     * was invoked.
     * </p>
     * 
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * <p>
     * If <code>act</code> is a CcActivity proxy and the associated project is
     * not CQ enabled, this operation simply executes the provided property
     * request -- if any.  If the project is CQ enabled, it additionally validates
     * that the bound CQ record is in an active state.
     * </p>
     * <p>
     * If <code>act</code> is a CqRecord proxy, this operation is more
     * involved.  First, <code>act</code> is automatically transitioned to an
     * active state.  If this transition involves required fields, the caller
     * may be prompted to provide values for those fields.  Next a CcActivity 
     * resource is created in this view's stream and is bound to
     * <code>act</code>.  
     * </p>
     * @param act the activity to work on - either a CqRecord or CcActivity
     * @param feedback optional property request
     * @return new proxy for the StpActivity, with requested properties
     * @see javax.wvcm.Workspace#CURRENT_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CC_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CQ_RECORD
     */
    public StpActivity doBindActivity(StpActivity act, Feedback feedback) throws WvcmException;

    /**
     * <p>Create a new ClearCase web view based on this CcView proxy.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>This proxy must have a file-based location representing the desired
     * path of the root directory of the file area on the local machine.
     * The directory must not yet exist.</li>
     * <li>The location's leaf name (the directory name) will be used as
     * the view tag of the new view.  A view having that view-tag must not
     * already exist.</li>
     * <li>To create a view associated with a UCM stream, specify that stream
     * by setting this view proxy's STREAM property.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A ClearCase web view is created, with the view tag
     * and local file area as described above.</li>
     * <li>If the view proxy's STREAM property was set, the view will be
     * associated with that stream.</li>
     * <li>If the view proxy's VIEW_TAG_STRING property was set, the view will
     * be assigned that tag.  Otherwise, the file area's root directory name
     * will be used.</li>
     * </ul>
     * @throws WvcmException if the preconditions are not met, or if there is an
     * error creating the view. 
     */
    public CcView doCreateCcWebView(Feedback feedback) throws WvcmException;

    /**
     * <p>Create a new ClearCase dynamic view based on this CcView proxy.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>The view proxy's VIEW_TAG_STRING property must be set
     * to the desired view tag of the new view.  A view having that view 
     * tag must not already exist.</li>
     * <li>The view proxy's VIEW_STORAGE_PATH property must be set
     * to the desired storage path for the new view unless a storage location
     * is being used.
     * <li>If a storage location is desired, rather than an explicit path.
     * then the VIEW_STORAGE_LOCATION property must be set to that storage
     * location.
     * <li>To create a view associated with a UCM stream, specify that stream
     * by setting this view proxy's STREAM property.</li>
     * <li>The proxy's location is ignored for creation.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A ClearCase dynamic view is created, with the view tag
     * and storage path as described above.</li>
     * <li>If the view proxy's STREAM property was set, the view will be
     * associated with that stream.</li>
     * 
     * </ul>
     * @throws WvcmException if the preconditions are not met, or if there is an
     * error creating the view. 
     */
    public CcView doCreateCcDynamicView(Feedback feedback) throws WvcmException;
    
    /**
     * <p>The rebase command reconfigures a stream by adding, dropping, or 
     * replacing one or more of the stream's foundation baselines. The file and
     * directory versions selected by those new baselines (and thus their 
     * associated activities) then become visible in the stream's views.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>The baseline is not from the stream that is being rebased.</li>
     * <li>The baseline is labeled. Baselines created by deliver operations are
     * not labeled by default.</li>
     * <li>Another rebase operation is not in progress.</li>
     * <li>A deliver operation is not in progress.</li>
     * </ul>
     * Additional rules apply to integration streams and development
     * streams in selecting a baseline:
     * <ul>
     * <li>An integration stream can be rebased only to a baseline created in 
     * another project or to an imported or initial baseline of that project.</li>
     * <li>A development stream can be rebased to a baseline that meets one 
     * of the following criteria:
     *      <ul>
     *      <li>The baseline was created in its parent stream.</li>
     *      <li>The baseline is in its parent stream's foundation.</li>
     *      <li>The baseline is an ancestor of the development stream's parent
     *       foundation baseline and created on the same stream as the parent's
     *       foundation baseline.</li>
     *      <li>The baseline was created in a stream other than its parent 
     *       stream and is contained in its parent stream. A baseline is 
     *       contained in another baseline if all changes in the first baseline
     *       are included in the second baseline.</li> 
     *      </ul>
     * </li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A rebase is started on the specified view. Use one of the other 
     * methods to resume, complete, or cancel the rebase.</li>
     * <li>The activity is set to the rebase activity.</li>
     * <li>If <code>baselineList</code> is specified and those baselines meet 
     * the preconditions above, the rebase will be started to those baselines.</li>
     * <li>If <code>baselineList</code> is not specified, the rebase will be 
     * started to the source stream's recommended baselines.</li>
     * <li>If <code>doAutoMerge</code> is true, elements needing merge will 
     * attempt to automatically merge on the server.</li> 
     * </ul>
     * 
     * @param baselineList optional list of baselines to include
     * @param doAutoMerge tells the rebase operation to attempt auto merges
     * @param integrationListener optional listener to get feedback on operation
     * @param updateListener optional listener to get feedback on view update 
     * changes
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * 
     * @throws WvcmException if there is an error starting the rebase operation
     */
    public CcView doStartRebase(
            List<CcBaseline> baselineList,
            boolean doAutoMerge,
            CcListener integrationListener,
            CcListener updateListener,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Restarts a rebase operation from the point at which it has been 
     * suspended. A rebase operation can be interrupted or when it encounters an
     * external error or condition that requires more information. However, you
     * cannot resume a rebase operation that has been successfully halted with
     * the <code>CcView.doCancelRebase()</code> operation.
     * </p>
     * 
     * @param doAutoMerge tells the rebase operation to attempt auto merges
     * @param integrationListener optional listener to get feedback on operation
     * @param updateListener optional listener to get feedback on view update 
     * changes
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error resuming the rebase operation
     */
    public CcView doResumeRebase(
            boolean doAutoMerge,
            CcListener integrationListener,
            CcListener updateListener,
            Feedback feedback) throws WvcmException;
    

    /**
     * <p>
     * Completes a rebase operation taking the following actions:
     * </p>
     * <ul>
     * <li>Resumes the rebase operation</li>
     * <li>Verifies that needed merges were made</li>
     * <li>Checks in any versions that are checked out</li>
     * <li>Records changes in the change set for the rebase activity</li>
     * </ul>
     * 
     * @param integrationListener optional listener to get feedback on operation
     * @param updateListener optional listener to get feedback on view update 
     * changes
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error completing the 
     * rebase operation
     */
    public CcView doCompleteRebase(
            CcListener integrationListener, 
            CcListener updateListener,            
            Feedback feedback) throws WvcmException;      
    
    /**
     * <p>
     * Cancels a rebase operation and restores the stream's previous 
     * configuration. Deletes the integration activity and any 
     * versions created by the rebase operation that are not yet checked in. 
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>None of the versions created in the integration activity can be
     * checked in.</li>
     * </ul>
     * 
     * @param integrationListener optional listener to get feedback on operation
     * @param updateListener optional listener to get feedback on view update 
     * changes
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error canceling the 
     * rebase operation
     */
    public CcView doCancelRebase(
            CcListener integrationListener, 
            CcListener updateListener,            
            Feedback feedback) throws WvcmException;    

    /**
     * <p>
     * The deliver command lets you deliver work from a source stream to a 
     * target stream in the same or a different project. This method delivers
     * all activities in the stream that have changed since the last deliver 
     * operation from the stream. The target stream is determined by the 
     * <code>CcView</code> integration view passed to this method.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started on the specified view. Use one of the other 
     * deliver methods to resume, complete, or cancel the deliver.</li>
     * <li>The deliver operation creates a UCM integration activity and sets 
     * it as the current activity in the integration view. This activity 
     * records the change set for the deliver operation.</li>
     * </ul>
     * 
     * @param integrationView the target view to deliver to
     * @param doAutoMerge tells the deliver operation to attempt auto merges
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     */
    public CcView doStartDeliver(
            CcView integrationView, 
            boolean doAutoMerge,
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException;    
  
    /**
     * <p>
     * Deliver the specified baselines from the source stream associated with 
     * this view to the target stream associated with the specified integration 
     * view. A development stream can deliver activities or baselines, but an
     * integration stream can deliver only baselines. Refer to the ClearCase
     * cleartool deliver man page for more details.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started on the specified view. Use one of the other 
     * deliver methods to resume, complete, or cancel the deliver.</li>
     * <li>The deliver operation creates a UCM integration activity and sets 
     * it as the current activity in the integration view. This activity 
     * records the change set for the deliver operation.</li>
     * </ul>
     * 
     * @param integrationView the target view to deliver to
     * @param baselineList list of baselines to deliver
     * @param doAutoMerge tells the deliver operation to attempt auto merges
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     */
    public CcView doStartDeliverBaselines(            
            CcView integrationView,
            ResourceList<CcBaseline> baselineList,
            boolean doAutoMerge,
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException; 
    
    /**
     * <p>
     * Deliver the specified activities from the source stream associated with 
     * this view to the target stream associated with the specified integration 
     * view. The list of activities must be self-consistent: the 
     * activities specified must not depend on the inclusion of any unspecified 
     * activities. Refer to the ClearCase cleartool deliver man page for more
     * details.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The list of activities must be self-consistent.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started on the specified view. Use one of the other 
     * deliver methods to resume, complete, or cancel the deliver.</li>
     * <li>The deliver operation creates a UCM integration activity and sets 
     * it as the current activity in the integration view. This activity 
     * records the change set for the deliver operation.</li>
     * </ul>
     * 
     * @param integrationView the target view to deliver to
     * @param activityList optional list of activities to deliver
     * @param doAutoMerge tells the deliver operation to attempt auto merges
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     * 
     */
    public CcView doStartDeliverActivities(            
            CcView integrationView,
            ResourceList<CcActivity> activityList,
            boolean doAutoMerge,
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException;   
    
    /**
     * <p>
     * In a MultiSite configuration where a team of developers works at a 
     * remote site, the project's integration stream may be mastered at a 
     * different replica than the developers' development streams. In this 
     * situation, the developers cannot complete deliver operations to the 
     * integration stream. When such a stream mastership situation is 
     * detected, the deliver operation starts but no versions are merged. 
     * Instead, the deliver operation is left in a posted state. You must 
     * periodically find and complete posted deliver operations. The post 
     * deliver command lets you do such the delivery work from a source 
     * stream to a target stream in the same or a different project. This 
     * method delivers all activities in the stream that have changed since 
     * the last deliver operation from the stream. The target stream is 
     * determined by the <code>Stream</code> targetStream passed to this 
     * method.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started but no versions are merged. Instead, the 
     * deliver operation is left in a posted state. You must periodically find
     * and complete posted deliver operations.</li>
     * </ul>
     * 
     * @param targetStream the target stream to deliver to
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     */
    public CcView doStartPostDeliver(
            Stream targetStream, 
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException;    
  
    /**
     * <p>
     * Post deliver the specified baselines from the source stream associated 
     * with this view to the target stream. A development stream can deliver 
     * activities or baselines, but an integration stream can deliver only 
     * baselines. Refer to the ClearCase cleartool deliver man page for more 
     * details.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started but no versions are merged. Instead, the 
     * deliver operation is left in a posted state. You must periodically find
     * and complete posted deliver operations.</li>
     * </ul>
     * 
     * @param targetStream the target stream to deliver to
     * @param baselineList list of baselines to deliver
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     */
    public CcView doStartPostDeliverBaselines(            
            Stream targetStream, 
            ResourceList<CcBaseline> baselineList,
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException; 
    
    /**
     * <p>
     * Post deliver the specified activities from the source stream associated 
     * with this view to the target stream associated with the specified 
     * integration view. The list of activities must be self-consistent: the 
     * activities specified must not depend on the inclusion of any unspecified 
     * activities. Refer to the ClearCase cleartool deliver man page for more
     * details.
     * </p>
     * <p>
     * Preconditions:
     * </p>
     * <ul>
     * <li>A deliver operation is not currently in progress.</li>
     * <li>A rebase operation is not currently in progress.</li>
     * <li>The list of activities must be self-consistent.</li>
     * <li>The checkout states must match the project's UCM policies.</li>
     * <li>Delivering to a non-default target stream is subject to 
     * restrictions. Refer to the ClearCase cleartool deliver man page
     * for details.</li>
     * </ul>
     * <p>
     * Postconditions:
     * </p>
     * <ul>
     * <li>A deliver is started but no versions are merged. Instead, the 
     * deliver operation is left in a posted state. You must periodically find
     * and complete posted deliver operations.</li>
     * </ul>
     * 
     * @param targetStream the target stream to deliver to
     * @param activityList optional list of activities to deliver
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error starting the deliver operation
     * 
     */
    public CcView doStartPostDeliverActivities(            
            Stream targetStream, 
            ResourceList<CcActivity> activityList,
            CcListener integrationListener, 
            Feedback feedback) throws WvcmException;  
    
    /**
     * <p>
     * Resumes a deliver operation from the point at which it was suspended. 
     * </p>
     * 
     * @param integrationView the target view to deliver to
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error resuming the deliver operation
     */    
    public CcView doResumeDeliver(
            CcView integrationView,
            CcListener integrationListener,
            Feedback feedback) throws WvcmException;    

    /**
     * <p>
     * Resets a deliver operation in progress to use the new integration view 
     * </p>
     * 
     * @param integrationView the new integration view
     * @param integrationListener optional listener to get feedback on operation
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error resuming the deliver operation
     */     
    public CcView doResetDeliver(
            CcView integrationView,
            CcListener integrationListener,
            Feedback feedback) throws WvcmException;    

    /**
     * <p>
     * Complete in-progress deliver operation taking the following actions:
     * </p>
     * <ul>
     * <li>Verify that all versions in the integration view have been 
     * successfully merged and that merge conflicts have been resolved.</li>
     * <li>Check in resulting versions in the integration view.</li>
     * <li>Unset the integration activity in the integration view.</li>
     * </ul>
     * <p>  
     * If unresolved merge conflicts exist, the deliver operation is suspended.
     * </p>
     * 
     * @param integrationView the target view to deliver to
     * @param integrationListener optional listener to get feedback on operation
     * @param mergeElements optional list of client maintained 
     * <code>CcMergeElement</code>s to allow the operation to cancel any 
     * checkouts
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error completing the deliver
     * operation
     */      
    public CcView doCompleteDeliver(
            CcView integrationView,
            CcListener integrationListener,
            CcMergeElement[] mergeElements,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Halt a deliver operation in progress, returning the source and 
     * destination streams to their states before the deliver operation began. 
     * Note: This method cannot cancel a deliver operation after the completion
     * phase has begun. See ClearCase cleartool deliver man page for details.
     * </p>
     * 
     * @param integrationView the target view to deliver to
     * @param integrationListener optional listener to get feedback on operation
     * @param mergeElements optional list of client maintained 
     * <code>CcMergeElement</code>s to allow the operation to cancel any 
     * checkouts
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error cancelling the deliver 
     * operation
     */
    public CcView doCancelDeliver(
            CcView integrationView,
            CcListener integrationListener,
            CcMergeElement[] mergeElements,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Runs findmerge command in this view. Searches the specified elements for merge candidates using
     * the specified VOB resource as a search criteria. The VOB resource can be either a branch type, or
     * a label type, or a version. Flags further define the search criteria. 
     * Found merge candidates are returned through the listener.
     * </p> 
     * 
     * @param listener optional listener to get feedback on operation. 
     * Note: although a listener is not required, that is the only way for a consumer of this method 
     * to receive the results of the findmerge operation.
     * @param flags array of flags which specify the behavior of the operation
     * @param fromVobResource VOB resource to use as a search criteria; Legal argument types include
     * CcBranchType, CcLabelType and CcVersion
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error completing the findmerge operation 
     *
     * @return new proxy to the view with requested properties
     * @throws WvcmException
     */
    public CcView doFindMergeCandidates(
            CcFindmergeListener listener,
            FindmergeFlag []flags,
            CcVobResource fromVobResource,
            List<CcFile> elements,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Runs findmerge command in this view. Searches the specified elements for merge candidates using
     * the specified View resource as a search criteria. Flags further define the search criteria. 
     * Found merge candidates are returned through the listener.
     * </p> 
     * 
     * @param listener optional listener to get feedback on operation. 
     * Note: although a listener is not required, that is the only way for a consumer of this method 
     * to receive the results of the findmerge operation.
     * @param flags array of flags which specify the behavior of the operation
     * @param fromViewTag View tag to use as a search criteria; 
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error completing the findmerge operation 
     *
     * @return new proxy to the view with requested properties
     * @throws WvcmException
     */
    public CcView doFindMergeCandidates(
            CcFindmergeListener listener,
            FindmergeFlag []flags,
            CcViewTag fromViewTag,
            List<CcFile> elements,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>
     * Runs findmerge command in this view. The specified activities are used as the search criteria.
     * Flags further define the search criteria. Found merge candidates are returned through the listener.
     *
     * </p> 
     * 
     * @param listener optional listener to get feedback on operation. 
     * Note: although a listener is not required, that is the only way for a consumer of this method 
     * to receive the results of the findmerge operation.
     * @param flags array of flags which specify the behavior of the operation
     * @param activities list of activities whose change sets to search for merge candidates
     * @param feedback optional property request
     * @return new proxy to the view with requested properties
     * @throws WvcmException if there is an error completing the findmerge operation 
     *
     */
    public CcView doFindMergeCandidatesFromChangeSets(
            CcFindmergeListener listener,
            FindmergeFlag []flags,
            List<CcActivity> activities,
            Feedback feedback) throws WvcmException;
    
    /**
     * <p>Upgrade this web view's file area.</p>
     * <p>Preconditions:</p>
     * <ul>
     * <li>This web view's file area schema version must be older than the
     * version supported by the currently running CM API file area management
     * code.
     * </li>
     * </ul>
     * <p>Postconditions:</p>
     * <ul>
     * <li>This web view's file area schema will be upgraded to the currently
     * supported version.</li>
     * </ul>
     * <p>This operation has no effect on non-web views, and on web views that
     * are already compatible.
     * </p>
     * @throws WvcmException if the preconditions are not met, or if there is an
     * error upgrading the file area. 
     */
    public CcView doUpgradeFileArea(Feedback feedback) throws WvcmException;

    /**
     * Work on the specified activity in this CC view.
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * <p>
     * If <code>act</code> is a CcActivity proxy and the associated project is
     * not CQ enabled, this operation simply makes it the current activity 
     * in this view.  If the project is CQ enabled, it additionally validates 
     * that the bound CQ record is in an active state.
     * </p>
     * <p>
     * If <code>act</code> is a CqRecord proxy, this operation is more
     * involved.  First, <code>act</code> is automatically transitioned to an
     * active state.  If this transition involves required fields, the caller
     * may be prompted to provide values for those fields.  Next a CcActivity 
     * resource is created in this view's stream and is bound to
     * <code>act</code>.  Finally, it makes the new CC activity resource the
     * current activity in this view.
     * </p>
     * @param act the activity to work on - either a CqRecord or CcActivity
     * @param feedback optional property request
     * @return new proxy for this view, with requested properties
     * @see javax.wvcm.Workspace#CURRENT_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CC_ACTIVITY
     * @see com.ibm.rational.wvcm.stp.StpActivity#BOUND_CQ_RECORD
     */
    public CcView doWorkOnActivity(StpActivity act, Feedback feedback) throws WvcmException;

    /**
     * <p>
     * Transitions the specified activity to the default completed state.
     * <code>act</code> can either be a CcActivity proxy or a CqRecord proxy.
     * </p>
     * <p>
     * NOTE: Unlike most API methods, the optional property request will be executed
     * on the returned StpActivity, not the CcView proxy on which the method
     * was invoked.
     * </p>
     * <p>Preconditions:</p>
     * <ul>
     * <li><code>act</code> is a bound CqRecord/CcActivity pair in a CQ-enabled
     * context.</li>
     * </ul>
     * <p>Postconditions:</p>
     * <ul>
     * <li>If <code>act</code>'s CcActivity has checkouts, the operation is
     * cancelled and there is no change to the activity.</li>
     * <li>Transitions <code>act</code>'s CqRecord to the default completed state. 
     * If this transition involves required fields, the caller may be prompted 
     * to provide values for those fields.</li>
     * <li>Unsets <code>act</code>'s CcActivity from all views'
     * {@link javax.wvcm.Workspace#CURRENT_ACTIVITY} property.</li>
     * </ul>
     * @param act the activity to finish
     * @param feedback optional property request for the activity
     * @return new proxy for the activity, with requested properties
     * @throws WvcmException if the preconditions are not met.
     */
    public StpActivity doFinishActivity(StpActivity act, Feedback feedback) 
        throws WvcmException;

    /**
     * <p>
     * Synchronize this web view file area's local databases to accurately reflect the 
     * current state of the file area. The databases available to synchronize are:
     * </p>
     * <ul>
     * <li>VOB database: This contains the loaded vobs in the web view. A server connection
     * is required to synchronize this database.</li>
     * <li>Modified files database: This contains the checkouts and hijacks in the web view.
     * A server connection <i>not</i> required to synchronize this database.</li>
     * </ul>
     * @param flags array of flags which specify the databases to synchronize.
     * @param feedback optional property request for the view
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException
     */
    public CcView doSynchronizeFileAreaDb(SynchronizeFileAreaDbFlag[] flags, Feedback feedback) 
        throws WvcmException;

    /**
     * Register this local web view's file area in the local file area registry.
     * Web views are registered automatically when created, but may need to be
     * re-registered if the registry is accidentally deleted, etc.
     */
    public void registerFileArea() throws WvcmException;

    /**
     * Remove this local web view's file area from the local file area registry.
     * Once removed, this view will no longer show up when listing local views.
     * @see com.ibm.rational.wvcm.stp.cc.CcProvider#getClientViewList(javax.wvcm.PropertyRequestItem.PropertyRequest)
     */
    public void unregisterFileArea() throws WvcmException;

    /**
     * If this is a local web view, the URL of the CCRC WAN server where this
     * web view's view database resides.
     */
    PropertyName<String> SERVER_URL =
        new PropertyName<String>(PROPERTY_NAMESPACE, "server-url");
   
    /**
     * Get the value of this proxy's {@link #SERVER_URL} property.
     * @return this view's server URL
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public String getServerUrl() throws WvcmException;

    /**
     * Change the URL used to connect to the CCRC WAN server on which this web view resides.
     * Note: Web views cannot be moved from one CCRC WAN server to another.
     * Use this method only to change the server URL, not to try to connect to
     * a different server.
     * <p>
     * This may be necessary if, for example:
     * <ul>
     * <li>The CCRC WAN server administrator changes the CCRC WAN server's port number</li>
     * <li>The client needs a secure HTTP connection to the CCRC WAN server:
     *     "https://..." instead of "http://..."</li>
     * <li>The CCRC WAN server is moved to a different internet subdomain</li>
     * </ul>
     * @param updatedUrl the updated URL of this web view's CCRC WAN server
     * @throws WvcmException
     */
    public void updateServerUrl(String updatedUrl) throws WvcmException;

    /**
     * This view's uuid as a string.
     */
    PropertyName<String> VIEW_UUID_STRING =
        new PropertyName<String>(PROPERTY_NAMESPACE, "view-uuid-string");
  
    /**
     * Get the value of this proxy's {@link #VIEW_UUID_STRING} property.
     * @return this view's uuid
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public String getViewUuidString() throws WvcmException;
    
    /**
     * This view's view tag as a string.
     */
    PropertyName<String> VIEW_TAG_STRING =
        new PropertyName<String>(PROPERTY_NAMESPACE, "view-tag-string");
  
    /**
     * Get the value of this proxy's {@link #VIEW_TAG_STRING} property.
     * @return this view's view tag
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public String getViewTagString() throws WvcmException;

    /**
     * Set the value of this view's {@link #VIEW_TAG_STRING} property.
     * This property may only be set at view creation time.
     * @param viewTag the view tag for the new view
     */
    void setViewTagString(String viewTag);
   
    /**
     * This view's view tag as a {@link CcViewTag} resource.
     */
    PropertyName<CcViewTag> VIEW_TAG =
        new PropertyName<CcViewTag>(PROPERTY_NAMESPACE, "view-tag");
   
    /**
     * Get the value of this proxy's {@link #VIEW_TAG} property.
     * @return this view's view tag
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public CcViewTag getViewTag() throws WvcmException;
   
    /** Kind of view to which this tag refers */
    PropertyName<ViewType> VIEW_TYPE =
        new PropertyName<ViewType>(PROPERTY_NAMESPACE,
                                   "view-view-type");
    
    /**
     * Returns the value  of this proxy's {@link #VIEW_TYPE} property.
     * 
     * @return Kind of view this tag refers to.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    ViewType getViewType() throws WvcmException;
    
    /**
     * Whereas a CcView resource's {@link javax.wvcm.Folder#CHILD_MAP} property
     * returns the root directories of <i>all</i> VOBs, LOADED_CHILD_MAP only
     * returns the root directories of VOBs that are partially or fully loaded
     * in this view (if this is a web view) or mounted (if this is a dynamic view).
     */
    PropertyName<Map<String,Resource>> LOADED_CHILD_MAP =
        new PropertyName<Map<String,Resource>>(PROPERTY_NAMESPACE, "loaded-child-map");
   
    /**
     * Get the value of this proxy's {@link #LOADED_CHILD_MAP} property.
     * @return this view's loaded child map
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    public Map<String,Resource> getLoadedChildMap() throws WvcmException;

    /**
     * This web view's file area root directory on the local machine.
     * The value of this property will be null if it does not have 
     * a file area on the local machine.
     * Only supported for web views.
     */
    PropertyName<File> FILE_AREA_ROOT_DIRECTORY =
        new PropertyName<File>(PROPERTY_NAMESPACE, "file-area-root-directory");
   
    /**
     * Returns the value of the {@link #FILE_AREA_ROOT_DIRECTORY} property.
     * @return This view's copy area root directory, or null if it has
     *         no copy area on the local machine
     * @throws WvcmException if this property is not defined by this proxy.
     */
    public File getFileAreaRootDirectory() throws WvcmException;

    /**
     * For a UCM view, the list of root directories for all components
     * included in that view's stream.
     * Always an empty list for non-UCM views.
     */
    PropertyName<ResourceList<CcDirectory>> COMPONENT_ROOT_DIRECTORY_LIST =
            new PropertyName<ResourceList<CcDirectory>>(PROPERTY_NAMESPACE, "component-root-directory-list");
    
    /**
     * Returns the value of the {@link #COMPONENT_ROOT_DIRECTORY_LIST} property.
     * @return List of the component root directories for this view's UCM stream, 
     *         or an empty list if this is not a UCM view.
     * @throws WvcmException if this property is not defined by this proxy.
     */
    public ResourceList<CcDirectory> getComponentRootDirectoryList() throws WvcmException;
    
    /**
     * Is this web view's local file area schema version older than the version
     * supported by the running CM API file management code? If so, the file
     * area needs to be upgraded before it can be used.
     * 
     * @see com.ibm.rational.wvcm.stp.cc.CcView#doUpgradeFileArea(Feedback)
     */
    PropertyName<Boolean> FILE_AREA_NEEDS_UPGRADE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "file-area-needs-upgrade");
   
    /**
     * Get the value of this view's {@link #FILE_AREA_NEEDS_UPGRADE} property.
     * @return true if this view's local file area needs upgrading; false otherwise.
     * @throws WvcmException if this property is not defined by this proxy.
     */
    boolean getFileAreaNeedsUpgrade() throws WvcmException;
   
    /**
     * Is this view associated with a UCM stream?.
     */
    PropertyName<Boolean> IS_UCM_VIEW =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-ucm-view");
   
    /**
     * Get the value of this view's {@link #IS_UCM_VIEW} property.
     * @return true if this view represents a UCM view; false otherwise.
     * @throws WvcmException if this property is not defined by this proxy.
     */
    boolean getIsUcmView() throws WvcmException;
   
    /**
     * This view's config spec.
     */
    PropertyName<CcConfigSpec> CONFIG_SPEC =
        new PropertyName<CcConfigSpec>(PROPERTY_NAMESPACE, "config-spec");
    
    /**
     * Get the value of this view's {@link #CONFIG_SPEC} property.
     * @return this view's config spec as a CcConfigSpec instance
     */
    CcConfigSpec getConfigSpec() throws WvcmException;

    /**
     * Set the value of this view's {@link #CONFIG_SPEC} property.
     * @param configSpec the new config spec for this view
     */
    void setConfigSpec(CcConfigSpec configSpec);
    
    /**
     * Network region in which the dynamic view is registered.
     * Not supported for other view types.
     * This is a write only property; to determine the region for
     * an existing view, use the CcViewTag.REGISTRY_REGION property.
     */
    PropertyName<CcRegistryRegion> REGION =
        new PropertyName<CcRegistryRegion>(PROPERTY_NAMESPACE, "view-region");
    
    /**
     * Set the value of this view's {@link #REGION} property. This
     * property may only be set at view creation time.
     * This is a write only property.
     * If unspecified, the local host's network region will be used.
     * @param region network region in which to register the view
     */
    void setRegion(CcRegistryRegion region);

    /**
     * Should derived objects created in this view be shared and made
     * available for winkin by other views?
     * (Dynamic views only, not supported for other view types.)
     */
    PropertyName<Boolean> SHARE_DERIVED_OBJECTS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "shared-dos");
    
    /**
     * Get the value of this view's {@link #SHARE_DERIVED_OBJECTS} property.
     * @return true if view creates shared derived objects, false otherwise
     */
    Boolean getShareDerivedObjects() throws WvcmException;

    /**
     * Set the value of this view's {@link #SHARE_DERIVED_OBJECTS} property. This
     * property may only be set at view creation time.
     * If unspecified, site-wide default is used.
     * If unspecified and no site-wide default, derived objects will be shared.
     * @param shareDerivedObjects true to make derived objects available for winkin
     * by other views, otherwise false
     */
    void setShareDerivedObjects(Boolean shareDerivedObjects);
    
    /**
     * The text mode of the view. The text mode specifies the line
     * terminator sequence for text files in the view. If no value is
     * set upon view creation, defaults to {@link TextMode#TRANSPARENT}.
     */
    PropertyName<TextMode> TEXT_MODE =
        new PropertyName<TextMode>(PROPERTY_NAMESPACE, "text-mode");
    
    /**
     * Get the value of this view's {@link #TEXT_MODE} property.
     * @return enumeration value representing this view's text mode.
     * @throws WvcmException if this proxy doesn't define a value for this property.
     */
    TextMode getTextMode() throws WvcmException;
    
    /**
     * Set the value of this view's {@link #TEXT_MODE} property. This
     * property may only be set at view creation time.
     * @param textMode the text mode of the view.
     */
    void setTextMode(TextMode textMode);
    
    /**
     * Break a file area lock on this web view with the given lock info.
     * @param lockInfo information about the lock
     * @return true if the lock was broken; false otherwise
     * @throws WvcmException if a problem occurred breaking the lock
     */
    boolean breakFileAreaLock(CcFileAreaLockInfo lockInfo) throws WvcmException;
    
    /**
     * CcViewAccessInfo object contains the supported view access properties.
     */
    public static final PropertyName<CcViewAccessInfo> VIEW_ACCESS_INFO =
        new PropertyName<CcViewAccessInfo>(PROPERTY_NAMESPACE, "view-access-info");

    /**
     * Returns the value of this proxy's {@link #VIEW_ACCESS_INFO} property.
     * 
     * @return the CcViewAccessInfo object.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcViewAccessInfo getViewAccessInfo() throws WvcmException;

    /**
     * The storage location for a dynamic view, not supported for other
     * view types.
     */
    public static final PropertyName<CcStorageLocation> VIEW_STORAGE_LOCATION =
        new PropertyName<CcStorageLocation>(PROPERTY_NAMESPACE, "view-stgloc");
    
    /**
     * Set the value of this view's {@link #VIEW_STORAGE_LOCATION} property.
     * This property may only be set at view creation time.
     * It is a write-only property.
     * @param viewStorageLocation the storage location for the new view
     */
    void setViewStorageLocation(CcStorageLocation viewStorageLocation);

    /**
     * The storage path for a dynamic view, not supported for other
     * view types.
     */
    public static final PropertyName<String> VIEW_STORAGE_PATH =
        new PropertyName<String>(PROPERTY_NAMESPACE, "view-storage-path");
    
    /**
     * Returns the value of this proxy's {@link #VIEW_STORAGE_PATH} property.
     * 
     * @return path to the view storage
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public String getViewStoragePath() throws WvcmException;
   
    /**
     * Set the value of this view's {@link #VIEW_STORAGE_PATH} property.
     * This property may only be set at view creation time.
     * @param viewStoragePath the storage path for the new view
     */
    void setViewStoragePath(String viewStoragePath);
      
    /**
     * <p>
     * The permissions applied to this resource.
     * </p>
     */
    public static final PropertyName<CcPermissions> PERMISSIONS =
        new PropertyName<CcPermissions>(PROPERTY_NAMESPACE, "cc-permissions");
    
    /**
     * Returns the value of this proxy's {@link #PERMISSIONS} property.
     * 
     * @return A permissions object from which specific permissions 
     * 	       information can be extracted.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public CcPermissions getPermissions() throws WvcmException;

    /**
     * Does this view have non-shareable derived objects?
     * This property is supported for dynamic views only.
     */
    public static final PropertyName<Boolean> IS_EXPRESS =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-express");
    
    /**
     * Returns the value of this proxy's {@link #IS_EXPRESS} property.
     * 
     * @return true if this view has non-shareable DOs; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getIsExpress() throws WvcmException;

    /**
     * Are this view's permissions valid?
     */
    public static final PropertyName<Boolean> ARE_PERMISSIONS_VALID =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "are-permissions-valid");
    
    /**
     * Returns the value of this proxy's {@link #ARE_PERMISSIONS_VALID} property.
     * 
     * @return true if this view's permissions are valid; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getArePermissionsValid() throws WvcmException;

    /**
     * Is this view read-only?
     */
    public static final PropertyName<Boolean> IS_READ_ONLY =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-read-only");
    
    /**
     * Returns the value of this proxy's {@link #IS_READ_ONLY} property.
     * 
     * @return true if this view is read-only; false otherwise.
     * @throws WvcmException
     *             if this proxy doesn't define a value for this property.
     */
    public Boolean getIsReadOnly() throws WvcmException;
    
     /**
     * Set the {@link #STREAM} property.
     * 
     * @param stream the {@link Stream} object that
     * identifies the {@link #STREAM} for this Workspace.
     * @see #getStream
     */
    public void setStream(Stream stream);

}
